/*-
 * Copyright (C)2021 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ident "@(#) Copyright (C)2021 @BABOLO http://www.babolo.ru/"
#ident "@(#) $Id: utils.c,v 1.9 2021/05/29 10:27:42 babolo Exp $"

#define BLIN_COMPAT     4
#define MIFE_COMPAT     5
#define Bpars_COMPAT    4
#define RECOBE_COMPAT   4

#include <sys/types.h>
#include <sys/event.h>
#include <sys/time.h>
#include <sysexits.h>
#include <strings.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <zlib.h>
#include <babolo/BLINflag.h>
#include <babolo/recobe.h>
#include <babolo/parser.h>
#include <mife.h>
#include "pngtr.h"

const u_char    pngtr_sg[] = PNGTR_SIGNATURE;

const u_int64_t pngtr_sigs[PNGTR_SIGNA_NUM + 1] =
{0x00556E6B00000000ULL, PNGTR_SIGNA_PNG, PNGTR_SIGNA_MNG, PNGTR_SIGNA_JNG};

const u_int8_t pngtr_pointsize[7][17] =
/*  0  1  2  3  4  5  6  7   8  9  0  1  2  3  4  5  6 */
{ { 0, 1, 2, 0, 4, 0, 0, 0,  8, 0, 0, 0, 0, 0, 0, 0, 16} /* 0                         */
, { 0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,  0} /* 1            palette ---- */
, { 0, 0, 0, 0, 0, 0, 0, 0, 24, 0, 0, 0, 0, 0, 0, 0, 48} /* 2      color              */
, { 0, 1, 2, 0, 4, 0, 0, 0,  8, 0, 0, 0, 0, 0, 0, 0,  0} /* 3      color palette      */
, { 0, 0, 0, 0, 0, 0, 0, 0, 16, 0, 0, 0, 0, 0, 0, 0, 32} /* 4 alfa                    */
, { 0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,  0} /* 5 alfa       palette ---- */
, { 0, 0, 0, 0, 0, 0, 0, 0, 32, 0, 0, 0, 0, 0, 0, 0, 64} /* 6 alfa color              */
};

static char
z(u_char u) {
    char s;

    if  (u < ' ') s = '.';
    else if ((u < 0x7F) || (u >= (u_char)'') || (u == (u_char)'') || (u == (u_char)'')) s = u;
    else s = '.';
    return(s);
}

int
/*****************************************************************************************************
 *****************************************************************************************************/
pngtr_dump(pngtr_conf *cf, const void *buf, ssize_t len, FILE *pr) {                             /****
 *****************************************************************************************************
 *****************************************************************************************************/
#   define blin_internal_flags (cf->flags)
    int               ex     = EX_OK;
    const u_char     *c;

    (void)cf;
    c = buf;
    while (len > 0) {
        fprintf( pr
               , ((int)(c - (u_char*)buf) < 0x010000) ? "      %04X" : "     %5X"
               , (int)(c - (u_char*)buf)
               );
        for (int i = 0; i < 16; ++i) {
            if  (i && !(i & 3)) fprintf(pr, " ");
            if  (i < len) {
                fprintf(pr, " %02X", c[i]);
            } else {
                fprintf(pr, "   ");
        }   }
        fprintf(pr, " |");
        for (int i = 0; i < 16; ++i) if  (i < len) fprintf(pr, "%c", z(c[i]));
        fprintf(pr, "|\n");
        if  (len >= 16) {
            c += 16;
            len -= 16;
        } else {
            c += len;
            len = 0;
    }   }
    return(ex);
#   undef blin_internal_flags
}

pngtr_miaobj *
/*****************************************************************************************************
 *****************************************************************************************************/
pngtr_minit(pngtr_conf *cf, u_int8_t size, int8_t width) {                                       /****
 *****************************************************************************************************
 *****************************************************************************************************/
#   define blin_internal_flags (cf->flags)
    pngtr_miaobj *minimax;

    if  (!(minimax = calloc(1 + size, sizeof(pngtr_miapair)))) {
        ifBLIN_QW0("calloc %u x %u", 1 + size, sizeof(pngtr_miapair));
    } else {
        minimax->size = size;
        minimax->width = width;
        for (u_int i = 0; i < size; ++i) minimax->minimax[i].min = 65535;
    }
    ifBLIN_QX3("- %"BLIN_X, minimax);
    return(minimax);
#   undef blin_internal_flags
}

int
/*****************************************************************************************************
 *****************************************************************************************************/
pngtr_minimax(pngtr_conf *cf, pngtr_miaobj *m, void *val) {                                      /****
 *****************************************************************************************************
 *****************************************************************************************************/
#   define blin_internal_flags (cf->flags)
    int        ex = EX_OK;
    u_int16_t  uval;
    u_int8_t  *v1 = val;

    switch(m->width) {
    case 1:
        for (int i = 0; i < m->size; ++i) {
            if  (m->minimax[i].min > v1[i]) m->minimax[i].min = v1[i];
            if  (m->minimax[i].max < v1[i]) m->minimax[i].max = v1[i];
        }
        break;
    case -2:
    case 2:
        for (int i = 0; i < m->size; ++i) {
            uval = recobe_load(&v1[2 * i], m->width);
            if  (m->minimax[i].min > uval) m->minimax[i].min = uval;
            if  (m->minimax[i].max < uval) m->minimax[i].max = uval;
        }
        break;
    }
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}
